from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import os
import sys
import pathlib
import time

from ImageCreator.core.make_template import select, to_dict, file_writer
from ImageCreator.core.image_create import safe_create
from ImageCreator.core import assets_directory


abspath = os.path.abspath(sys.argv[0])
dname = os.path.dirname(abspath)
os.chdir(dname)

config_dir = os.path.join(os.path.dirname(assets_directory), "CreateConfigurations")


def create(path_to_config=None, save_path=None, filename=None, return_image=False):
    if isinstance(path_to_config, str):
        if not os.path.exists(path_to_config):
            p = os.path.join(config_dir, path_to_config)
            if os.path.exists(p+".yml"):
                path_to_config = p+".yml"
            elif os.path.exists(p+".yaml"):
                path_to_config = p+".yaml"
            elif os.path.exists(p+".json"):
                path_to_config = p+".json"

    if save_path is None:
        save_path = os.path.join(config_dir, "created_images")
    if path_to_config is None:
        pathlist = []
        for f in os.listdir(config_dir):
            if f.endswith(".yaml") or f.endswith(".yml") or f.endswith(".json"):
                path = safe_create(os.path.join(config_dir, f), save_path=save_path, save_name=pathlib.Path(f).stem)
                pathlist.append(path)
        return pathlist
    if filename is None:
        filename = pathlib.Path(path_to_config).stem

    path = safe_create(path_to_config, save_path=save_path, save_name=filename, return_image=return_image)
    return path


def select_file():
    file_list = []
    for i in os.listdir(config_dir):
        if i.endswith(".json") or i.endswith(".yml") or i.endswith(".yaml"):
            file_list.append(i)

    print("All available configuration files:\n")
    for k, v in enumerate(file_list):
        print(f"{k}:    {v}")
    while True:
        try:
            num = input("\nPlease enter the number of your desired configuration (nothing=all): ")
            if num == "":
                return None, None
            num = int(num)
        except ValueError:
            print("It has to be a number!")
            continue
        if num not in range(len(file_list)):
            print("Number is not in range of file list!")
            continue
        else:
            break

    return os.path.join(config_dir, file_list[num]), pathlib.Path(file_list[num]).stem


def create_on_press():
    abs_path, filename = select_file()
    print("Process can be stopped with pressing CRTL + C")
    while True:
        try:
            input("Press enter for update... ")
            safe_create(abs_path, save_path=os.path.join(config_dir, "created_images"), save_name=filename)
        except KeyboardInterrupt:
            print("\n=== Create process ended ===\n")
            break


# TODO: implement create on watchdog
def create_on_modified():
    print("Listening for changes...")
    print("Process can be stopped with pressing CRTL + C")

    class MyHandler(FileSystemEventHandler):
        def on_modified(self, event):
            path = event.src_path
            if path.endswith(".yml") or path.endswith(".yaml") or path.endswith(".json"):
                safe_create(
                    event.src_path, save_path=os.path.join(config_dir, "created_images"),
                    save_name=pathlib.Path(event.src_path).stem
                )

            print(f"\n{pathlib.Path(event.src_path).stem} has been modified... ")

    event_handler = MyHandler()
    observer = Observer()
    observer.schedule(event_handler, path=config_dir, recursive=False)
    observer.start()

    try:
        while True:
            time.sleep(0.1)

    # TODO: make KeyboardInterrupt feature work
    except KeyboardInterrupt:
        observer.stop()
        exit(-1)


def create_template():
    json_ = to_dict(select())
    file_writer(json_, directory=config_dir)



import os
import watchdog.events
import watchdog.observers
import yaml

from tkinter import ttk
from tkinter import filedialog
from tkinter import colorchooser
from tkinter import *

from PIL import Image, ImageTk

from ImageCreator.core.image_create import CreatePicture, create_picture, safe_create
from ImageCreator.core.utils import to_json
from ImageCreator.core import assets_directory
from ImageCreator.core.use import create
from ImageCreator.core.make_template import base_image, overlay_image, text, multiline_text

config_dir = os.path.join(os.path.dirname(assets_directory), "CreateConfigurations")
config_path = ""

root = Tk()
root.title("ImageCreator")
root.minsize(370, 250)

image = None


class Handler(watchdog.events.PatternMatchingEventHandler):
    def __init__(self, file_name):
        # Set the patterns for PatternMatchingEventHandler
        watchdog.events.PatternMatchingEventHandler.__init__(self, patterns=[file_name], ignore_directories=True, case_sensitive=False)
    def on_modified(self, event):
        print(">>> modified")
        try:
            refresh_image()
        except Exception as e:
            print(f"( {e} )")


def update_credentials():
    # Update credentials
    global image
    w, h = image.size
    file_size = w * h * 3
    unit = "B"
    if file_size >= 1000000000:
        file_size /= 1000000000
        unit = "GB"
    elif file_size >= 1000000:
        file_size /= 1000000
        unit = "MB"
    elif file_size >= 1000:
        file_size /= 1000
        unit = "KB"
    credentials = f"{os.path.basename(config_path)}   {w}x{h}   {file_size:.2f} {unit}"
    credentials_label.configure(text=credentials)


def add_overlay():
    if not config_path:
        f = filedialog.asksaveasfile(initialdir=config_dir, mode='w', defaultextension=".yaml")
        if not f: return
        yaml.dump(base_image, f, default_flow_style=False, indent=4, sort_keys=False)
        f.close()

        get_config_path(f.name)
        return


    image_synonyms = ["image", "img"]
    text_synonyms = ["text", "txt"]
    multiline_text_synonyms = ["multiline_text", "multiline", "multi", "mtxt"]

    configs = to_json(config_path)

    def rec_loop(dict_):
        for k, v in dict_.items():
            if isinstance(v, dict):
                if k == "overlays":
                    for k_, v_ in v.items():
                        if v_ in image_synonyms:
                            v[k_] = overlay_image
                        elif v_ in text_synonyms:
                            v[k_] = text
                        elif v_ in multiline_text_synonyms:
                            v[k_] = multiline_text

                rec_loop(v)


    rec_loop(configs)
    print(configs)

    with open(config_path, "w") as f:
        yaml.dump(configs, f, default_flow_style=False, indent=4, sort_keys=False)


def image_show(image_, max_width=500, max_height=500):
    root.minsize(max_width, max_height+60)
    root.geometry(f"{max_width+20}x{max_height+60}")
    opening = CreatePicture().resize_img(image_, max_width, max_height)
    im = ImageTk.PhotoImage(opening)
    image_label.configure(image=im)
    image_label.image = im
    return im


def refresh_image():
    global config_path
    global image
    image = create(config_path, return_image=True)
    image_show(image)


def toggle_stay_on_top():
    root.attributes('-topmost', on_top.get())
    root.update()



observer = None

def toggle_auto_refresh():
    global config_path
    global observer
    print(config_dir, os.path.basename(config_path))
    

    if auto_refresh_value.get():
        observer = watchdog.observers.Observer()
        event_handler = Handler(os.path.basename(config_path)) 
        observer.schedule(event_handler, path=config_dir, recursive=False)

        observer.start()

    else:
        observer.stop()
        observer.join()
        observer = None


def get_config_path(path_=None):
    global config_path
    if path_:
        config_path = path_
    else:
        config_path = filedialog.askopenfilename(initialdir=config_dir, title="Select a File", filetypes=(
            ("Configuration Files", "*.yml *.yaml *.json"), ("All Files", "*.*")))
    if not config_path: return
    add_overlay_button.configure(text="Add Overlay")
    refresh_image()
    update_credentials()



on_top = BooleanVar()
auto_refresh_value = BooleanVar()

options_frame = ttk.Frame(root)
credentials_label = ttk.Label(root)
stay_on_top = ttk.Checkbutton(options_frame, variable=on_top, command=toggle_stay_on_top, text="Stay on top")
auto_refresh = ttk.Checkbutton(options_frame, variable=auto_refresh_value, command=toggle_auto_refresh, text="Auto refresh")
get_path_button = ttk.Button(options_frame, command=get_config_path, text="Select file")
refresh_image_button = ttk.Button(options_frame, command=lambda: refresh_image(), text="Refresh")
add_overlay_button = ttk.Button(options_frame, command=lambda: add_overlay(), text="Add File")

image_label = ttk.Label(root)


options_frame.place(anchor=N, relx=0.5)

credentials_label.place(anchor=S, rely=1, relx=0.5)
get_path_button.grid(row=0, column=1, padx=5, rowspan=2)
stay_on_top.grid(row=0, column=2, padx=5, sticky="W")
auto_refresh.grid(row=1, column=2, padx=5, sticky="W")
refresh_image_button.grid(row=0, column=3, padx=5, rowspan=2)
add_overlay_button.grid(row=0, column=4, padx=5, rowspan=2)
image_label.place(anchor=S, relx=0.5, rely=1, y=-15)


root.mainloop()


import yaml
import os
import importlib.util

from ImageCreator.core import assets_directory


def to_json(path):
    with open(path, "r") as yml_in:
        json_out = yaml.safe_load(yml_in)
    return json_out


def plugin_handler(value, plugin_directory):
    """
    Takes in plugins in the plugin format and recursively evaluates them\n
    Plugin format: {"plugin": "PLUGINNAME", "kwargs": {"ANY": "THING"}}

    :param value: dict
    :param plugin_directory: str
    :return evaluated plugin: Any
    """
    if not isinstance(value, dict):
        return value
    elif value.get("plugin", None) is not None:
        plugin = value.get("plugin")

        kwargs_ = value.get("kwargs", {})
        for index, item in kwargs_.copy().items():
            if isinstance(item, dict):
                kwargs_[index] = plugin_handler(item, plugin_directory)

        try:
            spec = importlib.util.spec_from_file_location(plugin, os.path.join(plugin_directory, plugin + ".py"))
            plg = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(plg)
            run = f"plg.run(**{kwargs_})"
            val = eval(run)
        except Exception as e:
            print(f"Something in the plugin {plugin} went wrong:")
            print(f"\033[91m{e}\033[0m")
            val = None
        return val
    else:
        return value


def dict_iterator(input_dict: dict, plugin_directory: str):
    """
    Iterates through dictionary and uses plugin_handler to evaluate plugins

    :param plugin_directory: str:
    :param input_dict: dict:
    :return output_dict: dict:
    """
    output_dict = input_dict
    for k, v in output_dict.items():
        if isinstance(v, dict):
            if v.get("plugin", None) is not None:
                output_dict[k] = plugin_handler(v, plugin_directory)
            else:
                dict_iterator(v, plugin_directory)

    return output_dict



import os
import traceback
from PIL import Image, ImageEnhance, ImageFont, ImageDraw
import time
import sys

from ImageCreator.core.utils import *
from ImageCreator.core import assets_directory

sys.path.append(assets_directory)

try:
    from Assets.fonts._fonts import *
except ModuleNotFoundError:
    pass


class CreatePicture:
    def __init__(self, attrs=None):
        self.picture = None
        self.attrs = attrs

    def open_image(self, path_):
        pic = Image.open(path_)
        rgba_pic = pic.convert('RGBA')
        self.picture = rgba_pic
        return rgba_pic

    def set_background(self, background_picture=None, color=None, size=None):
        bgp = background_picture
        if bgp is None:
            self.picture = Image.new('RGBA', size, color)
        elif size is None:
            self.picture = bgp
        elif color is None:
            self.picture = bgp.resize(size, Image.ANTIALIAS)
        return self.picture

    def crop_(self, width=0, height=0, x=0, y=0, pic=None, to_square=False):
        if pic is None:
            im = self.picture
        else:
            im = pic

        if width == 0 or width is None:
            width = im.size[0]
        if height == 0 or height is None:
            height = im.size[1]

        if not to_square:
            cropped_image = im.crop((x, y, width, height))
        else:
            if width > height:
                crop = int((width - height) / 2)
                cropped_image = im.crop((crop + x, 0 + y, width - crop + x, height + y))  # (x, y, width, height)
            elif width < height:
                crop = int((height - width) / 2)
                cropped_image = im.crop((0 + x, crop + y, width + x, height - crop + y))
            else:
                cropped_image = im

        self.picture = cropped_image
        return cropped_image

    def overlay_images(self, bgi=None, fgi=None, posx=0, posy=0, anchor_center=True):
        if isinstance(fgi, str):
            overlay_ = Image.open(fgi)
            overlay = overlay_.convert('RGBA')
        else:
            overlay = fgi.convert('RGBA')

        if bgi is None:
            bg = self.picture
        else:
            if isinstance(bgi, str):
                bgi = Image.open(bgi)
            bg = bgi.convert("RGBA")

        comp_img = Image.new('RGBA', bg.size, (0, 0, 0, 0))

        if anchor_center:
            position = ((comp_img.width - overlay.width) // 2 + posx, (comp_img.height - overlay.height) // 2 + posy)
        else:
            position = (posx, posy)

        comp_img.paste(bg, (0, 0))  # ((comp_img.width - bg.width) // 2, (comp_img.height - bg.height) // 2))
        comp_img.paste(overlay, position, mask=overlay)

        if bgi is None:
            self.picture = comp_img

        return comp_img

    def resize_img(self, im=None, width=None, height=None, keep_aspect_ratio=True):
        if im is None:
            image = self.picture
        elif isinstance(im, str):
            image = Image.open(im)
        else:
            image = im

        width = width
        height = height

        if keep_aspect_ratio and width and height:
            w, h = image.size
            if w < h:
                height = None
            else:
                width = None

        if (width is None) and (height is None):
            output_img = image
        elif height is None:
            wpercent = (width / float(image.size[0]))
            hsize = int((float(image.size[1]) * float(wpercent)))
            output_img = image.resize((width, hsize), Image.ANTIALIAS)
        elif width is None:
            hpercent = (height / float(image.size[1]))
            wsize = int((float(image.size[0]) * float(hpercent)))
            output_img = image.resize((wsize, height), Image.ANTIALIAS)

        elif keep_aspect_ratio:
            image.thumbnail((width, height))
            output_img = image
        else:
            output_img = image.resize((width, height), Image.ANTIALIAS)

        if im is None:
            self.picture = output_img
        return output_img

    def add_text(
            self, message, size, font=None, posx=0, posy=0, color=(255, 255, 255, 255),
            image=None, anchor_center=True, **kwargs
    ):
        if image is None:
            txt_image = self.picture
        else:
            txt_image = image

        if message is None:
            return

        if isinstance(color, dict):
            rgba_tuple = (color.get("r", 255), color.get("g", 255), color.get("b", 255), color.get("a", 255))
            color = rgba_tuple

        if isinstance(font, str):
            if not font.endswith(".ttf"):
                font = eval(font)

        txt = Image.new("RGBA", txt_image.size, (255, 255, 255, 0))
        size_ = size
        while True:
            if font is None:
                fnt = ImageFont.load_default()
            elif font.endswith(".ttf"):
                fnt = ImageFont.truetype(font, size_)
            elif font.endswith(".otf"):
                fnt = ImageFont.truetype(font, size_)
            else:
                fnt = ImageFont.load_default()

            d = ImageDraw.Draw(txt)

            wi, he = (txt_image.size[0], txt_image.size[1])
            w, h = d.textsize(message, font=fnt)

            if w > 1050:
                size_ = size_ - 5   # higher = faster but more inaccurate ; lower = slower but more accurate
            else:
                break

        if anchor_center:
            d.text(((wi - w) / 2 + posx, (he - h) / 2 + posy), message, font=fnt, fill=color, **kwargs)
        else:
            d.text((posx, posy), message, font=fnt, fill=color, **kwargs)
        img_with_text = Image.alpha_composite(txt_image, txt)

        if image is None:
            self.picture = img_with_text
        return img_with_text

    def add_multiline_text(
            self, message, box_width, box_height=5000, size=30, font=None, posx=0, posy=0,
            color=(255, 255, 255), image=None, spacing=10, align="left", anchor_center=True, **kwargs
    ):
        if image is None:
            txt_image = self.picture
        else:
            txt_image = image

        if isinstance(color, dict):
            rgba_tuple = (color.get("r", 255), color.get("g", 255), color.get("b", 255), color.get("a", 255))
            color = rgba_tuple

        if isinstance(font, str):
            if not font.endswith(".ttf"):
                font = eval(font)

        txt = Image.new("RGBA", txt_image.size, (255, 255, 255, 0))
        size_ = size
        while True:
            if font is None:
                fnt = ImageFont.load_default()
            elif font.endswith(".ttf"):
                fnt = ImageFont.truetype(font, size_)
            elif font.endswith(".otf"):
                fnt = ImageFont.truetype(font, size_)
            else:
                fnt = ImageFont.load_default()

            d = ImageDraw.Draw(txt)

            word_list = message.split(' ')
            size_list = []
            bias = 0
            for i in word_list:
                w, h = d.textsize(i + ' ', font=fnt)
                size_list.append(w)

            if ':' in message:
                for i in range(len(word_list)):
                    if ':' in word_list[i]:
                        word_list[i + 1] = '\n' + word_list[i + 1]
                        bias = i

            colon_pos = bias
            for i in range(colon_pos, len(word_list)):
                if sum(size_list[bias:i + 1]) > box_width:
                    word_list[i] = '\n' + word_list[i]
                    bias = i
            formatted_text = ' '.join(word_list)

            wid, hei = d.multiline_textsize(formatted_text, font=fnt, spacing=spacing)
            if hei > box_height or wid > box_width:
                size_ -= 5  # higher = faster but more inaccurate ; lower = slower but more accurate
            else:
                break

        w, h = d.multiline_textsize(formatted_text, font=fnt, spacing=spacing)
        wi, he = (txt_image.size[0], txt_image.size[1])
        if anchor_center:
            d.multiline_text(((wi - w) / 2 + posx, (he - h) / 2 + posy), formatted_text, font=fnt, fill=color,
                             spacing=spacing, align=align, **kwargs)
        else:
            d.multiline_text((posx, posy), formatted_text, font=fnt, fill=color,
                             spacing=spacing, align=align, **kwargs)

        img_with_text = Image.alpha_composite(txt_image, txt)

        if image is None:
            self.picture = img_with_text
        return img_with_text

    def enhancer(self, image=None, brightness=1.0, color=1.0, contrast=1.0, sharpness=1.0):
        if image is None:
            img = self.picture
        elif isinstance(image, str):
            img = Image.open(image)
        else:
            img = image
        if brightness != 1.0:
            img = ImageEnhance.Brightness(img).enhance(brightness)  # 0.0 = black image ; 1.0 = original image
        if color != 1.0:
            img = ImageEnhance.Color(img).enhance(color)  # 0.0 = black and white image ; 1.0 = original image
        if contrast != 1.0:
            img = ImageEnhance.Contrast(img).enhance(contrast)  # 0.0 = solid grey image ; 1.0 = original image
        if sharpness != 1.0:
            img = ImageEnhance.Sharpness(img).enhance(sharpness)  # 0.0 = blurred ; 2.0 = sharpened

        if image is None:
            self.picture = img
        return img


example_params = {
    "path": "t1.jpg",
    "size": {"width": 1080, "height": 1080, "keep_aspect_ratio": True},
    "crop": {"x": 200, "y": 200, "width": 0, "height": 0, "to_square": True},
    "color": "#554477",
    "enhance": {"brightness": 1.0, "color": 1.0, "contrast": 1.0, "sharpness": 1.0},
    "opacity": 1.0,
    "overlays": {
        "1": {"type": "image", "location": {"posx": -300, "posy": -200, "anchor_center": True},
              "params": {
                  # "path": "t2.jpg",
                  "size": {"width": 200, "height": 200, "keep_aspect_ratio": True},
                  "crop": {"x": 0, "y": 0, "width": 60, "height": 70, "to_square": False},
                  "color": "#554477",
                  "enhance": {"brightness": 1.0, "color": 1.0, "contrast": 1.0, "sharpness": 1.0},
                  "opacity": 0.7
              }
              },
        "2": {"type": "text", "location": {"posx": 0, "posy": 0, "anchor_center": True},
              "params": {
                  "message": {"plugin": "get_timestamp", "kwargs": {"number": 400}},
                  "size": 70,
                  "font": "Anton_Regular",
                  "color": "#ffffff"
              }
              },
        "3": {"type": "multiline_text", "location": {"posx": 0, "posy": 100, "anchor_center": True},
              "params": {
                  "message": "Lorem ipsum dolor",
                  "box_width": 300,
                  "box_height": 300,
                  "size": 40,
                  "font": "Anton_Regular",
                  "color": "#eeee77",
                  "spacing": 10,
                  "align": "left"
              }
              }
    }
}


def create_picture(parameters, save_path=None, save_name=None, return_image=False):

    if isinstance(parameters, str):
        if os.path.exists(parameters):
            parameters = to_json(parameters)

    def image_handler(parameters_):

        img_dict = dict_iterator(parameters_, os.path.join(assets_directory, 'plugins'))

        create = CreatePicture(img_dict)

        path_ = img_dict.get("path", None)
        size_ = img_dict.get("size", None)
        crop_ = img_dict.get("crop", None)
        color_ = img_dict.get("color", None)
        enhance_ = img_dict.get("enhance", None)
        opacity_ = img_dict.get("opacity", None)
        overlays_ = img_dict.get("overlays", None)

        if path_ is not None:
            try:
                create.open_image(path_)
            except FileNotFoundError:
                filepath = os.path.join(assets_directory, path_)
                create.open_image(filepath)
            if size_ is not None: create.resize_img(**size_)
            # Color of image
            if color_ is not None: create.overlay_images(
                fgi=Image.new("RGBA", create.picture.size, color_))
        elif size_ is not None:
            size = (size_.get("width", 1), size_.get("height", 1))
            create.set_background(color=color_, size=size)

        if crop_ is not None:
            create.crop_(**crop_)

        if opacity_ is not None:
            create.picture.putalpha(int(255 * float(opacity_)))

        if enhance_ is not None:
            create.enhancer(**enhance_)

        if overlays_ is not None:
            overlay_items = overlays_.items()
            for index, overlay in overlay_items:
                if isinstance(overlay, str): continue
                overlay_type = overlay.get("type", None)
                if overlay_type == "image":
                    create.overlay_images(
                        fgi=image_handler(overlay["params"]), **overlay["location"])
                elif overlay_type == "text":
                    if overlay.get("box_width") or overlay.get("box_height") or overlay.get("spacing") or overlay.get("align"):
                        # if true text is multiline
                        params_ = {}
                        for i, v in overlay["params"].items():
                            params_[i] = v
                        create.add_multiline_text(**params_, **overlay["location"])
                    else:
                        params_ = {}
                        for i, v in overlay["params"].items():
                            params_[i] = v
                        create.add_text(**params_, **overlay["location"])
                elif overlay_type == "multiline_text":
                    params_ = {}
                    for i, v in overlay["params"].items():
                        params_[i] = v
                    create.add_multiline_text(**params_, **overlay["location"])
        return create.picture

    img_done = image_handler(parameters)
    if save_name:
        if save_name.endswith(".png"):
            name = save_name
        else:
            name = save_name + ".png"
    else:
        name = str(time.time()).split(".")[0] + ".png"

    if save_path is not None:
        try:
            path = os.path.join(save_path, name)
            with open(path, "w"):
                img_done.save(path, "PNG")
        except Exception as e:
            print(e)

    if return_image: return img_done
    else: return path



def safe_create(parameters, save_path=None, save_name=None, return_image=False):
    try:
        path = create_picture(parameters, save_path, save_name, return_image)
        return path
    except KeyboardInterrupt:
        exit(-1)
    except Exception as e:
        traceback.print_exc()
        print(e)



import json
import string
import yaml
import os


# TODO: create better base templates
base_image = {
             "path": None,
             "size": {"width": 1080, "height": 1080, "keep_aspect_ratio": True},
             "crop": {"x": 0, "y": 0, "width": 1080, "height": 1080, "to_square": False},
             "color": "#554477",
             "enhance": {"brightness": 1.0, "color": 1.0, "contrast": 1.0, "sharpness": 1.0},
             "opacity": 1.0,
             "overlays": {}
         }

overlay_image = {"type": "image", "location": {"posx": 0, "posy": 0, "anchor_center": True},
         "params": {
             "path": "t2.jpg",
             "size": {"width": 400, "height": 400, "keep_aspect_ratio": True},
             "crop": {"x": 0, "y": 0, "width": 400, "height": 400, "to_square": False},
             "color": "#554477",
             "enhance": {"brightness": 1.0, "color": 1.0, "contrast": 1.0, "sharpness": 1.0},
             "opacity": 1.0,
             "overlays": {}
         }}

text = {"type": "text", "location": {"posx": 0, "posy": -80, "anchor_center": True},
        "params": {
            "message": "Lorem ipsum",
            "size": 40,
            "font": "Anton_Regular",
            "color": "#000000"
        }
        }

multiline_text = {"type": "multiline_text", "location": {"posx": 50, "posy": 80, "anchor_center": True},
                  "params": {
                      "message": "Lorem ipsum dolor sit amet consectetur adipisicing elit.",
                      "box_width": 300,
                      "box_height": 300,
                      "size": 40,
                      "font": "Anton_Regular",
                      "color": "#000000",
                      "spacing": 10,
                      "align": "left"
                  }
                  }

global rec

rec = 0


# TODO: make structure better
# TODO: make items deletable
def select():
    global rec
    indent = ""
    overlays = []
    for i in range(rec):
        indent += " ---- "
    rec += 1
    print("\n" + "depth: " + str(rec))
    while True:
        print(indent + "i = image ; t = text ; m = multiline text ; n = none")
        inp = input(indent + "Select overlay: ")
        type_ = inp.lower()
        if type_ == "i":
            overlays.append(select())
        elif type_ in ["t", "m"]:
            overlays.append(type_)
        elif type_ == "n":
            print("")
            break
        else:
            print(indent + "Invalid input\n")

    return overlays


def to_dict(user_selection):
    def iterator(data):
        i = 0
        overlay_data = {}
        for ind in data:
            i += 1
            if isinstance(ind, list):
                sth = overlay_image
                overlay_data[str(i)] = sth
                overlay_data[str(i)]["params"]["overlays"] = json.loads(iterator(ind))

            elif ind == "t":
                sth = text
                overlay_data[str(i)] = sth
            elif ind == "m":
                sth = multiline_text
                overlay_data[str(i)] = sth

        ret = json.dumps(overlay_data, indent=4)
        return ret

    overlays = json.loads(iterator(user_selection))
    base_image["overlays"] = overlays
    return base_image


def file_writer(data, directory=""):
    while True:
        name_inp = input("Choose a filename: ")
        for l in name_inp:
            if l not in "-_.()" + string.ascii_letters + string.digits:
                print("Invalid input\n")
                continue
        break

    while True:
        format_inp = input("Choose a format (y=yaml, j=json): ").lower()
        if format_inp == "y" or format_inp == "yaml" or format_inp == "yml":
            filename = name_inp + ".yaml"

            while True:
                flow = input("Choose flow style (Enter=deafult , n=none , f=full)")
                if flow == "":
                    style = None
                    break
                elif flow == "n":
                    style = False
                    break
                elif flow == "f":
                    style = True
                    break
                else:
                    print("Invalid input\n")

            with open(os.path.join(directory, filename), "w") as f:
                f.write(yaml.dump(data, default_flow_style=style, indent=4, sort_keys=False))
            break

        elif format_inp == "j" or format_inp == "json":
            filename = name_inp + ".json"

            with open(filename, "w") as f:
                f.write(json.dumps(data, indent=4))
            break

        else:
            print("Invalid input\n")



import platform
import os


# When the package is a pip package, this can be removed
import sys
import pathlib
abspath = os.path.abspath(sys.argv[0])
dname = os.path.dirname(abspath)
sys.path.append(os.path.dirname(dname))
sys.path.append(str(pathlib.Path(__file__).parent.absolute()))
#

from ImageCreator.core.use import create, create_on_press, create_on_modified, create_template


def create_from_params(params_path, save_path=None, save_name=None):
    return create(params_path, save_path, save_name)


def start():
    import GUI


import os
import sys
import time


# When the package is a pip package, this can be removed
import sys
import pathlib
abspath = os.path.abspath(sys.argv[0])
dname = os.path.dirname(abspath)
sys.path.append(os.path.dirname(dname))
sys.path.append(str(pathlib.Path(__file__).parent.absolute()))
#

def make_dir(name):
    if not os.path.exists(name):
        os.mkdir(name)


def get_fonts():
    with open(os.path.join(assets_directory, "fonts", "_fonts.py"), 'w') as file:
        for root, dirs, files in os.walk(os.path.join(assets_directory, "fonts")):
            for filename in files:
                if filename.endswith(".ttf") or filename.endswith(".otf"):
                    new_root = root.replace(".", "")
                    fullpath = os.path.join(assets_directory, "fonts", new_root, filename)
                    new_filename = filename[:-4].replace('-', '_').replace(' ', '_').replace('.', '_')
                    line = f"{new_filename} = r'{fullpath}'\n"
                    file.write(line)


abspath = os.path.abspath(sys.argv[0])
dname = os.path.dirname(abspath)
os.chdir(dname)

make_dir("Assets")
make_dir("Assets/fonts")
make_dir("Assets/plugins")
make_dir("CreateConfigurations")
make_dir("CreateConfigurations/created_images")

assets_directory = os.path.abspath("Assets")

get_fonts()



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name='ImageCreator',
    version='0.0.2',
    description='A software to create images from a yaml template',
    packages=['ImageCreator', 'ImageCreator.core'],
    package_dir={'': '.'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent"

    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires = [
        "pyyaml >= 5.1"
    ],
    extras_require = {
        "dev": [
            "pytest>=3.7",
        ],
    },
    url="https://github.com/enocyzed/ImageCreator",
    author="Enoch",
    author_email="enocyzed@gmail.com",
    license="Apache License 2.0",

)